/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.core;

import java.util.ArrayList;
import java.util.List;

import org.ow2.petals.registry_overlay.api.Constants;
import org.ow2.petals.registry_overlay.configuration.generated.Credentials;
import org.ow2.petals.registry_overlay.configuration.generated.ManagementConsole;
import org.ow2.petals.registry_overlay.configuration.generated.Member;
import org.ow2.petals.registry_overlay.configuration.generated.Members;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;

import com.hazelcast.client.config.ClientConfig;
import com.hazelcast.client.config.ClientNetworkConfig;
import com.hazelcast.config.GroupConfig;

/**
 * A builder creating cluster configuration
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ClusterBuilder {

    protected final static String MEMBER_ID = "memberId";

    protected final static String CREDENTIALS_GROUP = "my-petals-esb-registry";

    protected final static String CREDENTIALS_PWD = "my password of my petals esb registry";

    private ClusterBuilder() {
        // No constructor because utility class
    }

    /**
     * Create a very basic cluster configuration:
     * <ul>
     * <li>only one member (identifier: {@link #MEMBER_ID},</li>
     * <li>management console is enable on the default URL,</li>
     * <li>using the default TCP port</li>
     * <li>security is based on {@link #CREDENTIALS_GROUP} and {@link #CREDENTIALS_PWD}</li>
     * </ul>
     * 
     * @param hostname
     *            The hostname of the member
     * @return The very basic cluster configuration
     */
    protected static PetalsRegistryOverlay createVeryBasicClusterConfig(final String hostname) {
        final Member member = new Member();
        member.setId(MEMBER_ID);
        member.setValue(hostname);
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setGroup(CREDENTIALS_GROUP);
        credentials.setPassword(CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final ManagementConsole mngtConsole = new ManagementConsole();
        mngtConsole.setValue(RegistryOverlayNode.DEFAULT_MNGT_CENTER_URL);
        mngtConsole.setEnable(Boolean.TRUE);
        petalsRegistryOverlay.setManagementConsole(new ManagementConsole());

        return petalsRegistryOverlay;
    }

    /**
     * Create the HZL client configuration to join a cluster created through
     * {@link #createVeryBasicClusterConfig(String)}
     * 
     * @param hostname
     *            The name of the host on which the cluster is running
     * @return The HZL client configuration
     */
    protected static ClientConfig createClientConfigForVeryBasicCluster(final String hostname) {

        final ClientConfig hzlClientConfig = new ClientConfig();
        hzlClientConfig
                .setGroupConfig(new GroupConfig(ClusterBuilder.CREDENTIALS_GROUP, ClusterBuilder.CREDENTIALS_PWD));
        final ClientNetworkConfig networkConfig = new ClientNetworkConfig();
        final List<String> addresses = new ArrayList<String>();
        addresses.add(hostname + ":" + Constants.DEFAULT_PORT);
        networkConfig.setAddresses(addresses);
        hzlClientConfig.setNetworkConfig(networkConfig);

        return hzlClientConfig;

    }

}
